import os;
from os.path import isfile, isdir, join, abspath

class DirCrawler:

	@classmethod
	def stdpath(self, path):
		platform = os.name
		if platform == 'nt':
			result = abspath(path).replace('\\','/')
			if result[0:5] == 'C:/c/': return result.replace('C:/c/','C:/')
			return result
		else:
			return abspath(path)

	@classmethod
	def joinpath(self, path, filename):
		platform = os.name
		if platform == 'nt':
			result = abspath(join(path,filename)).replace('\\','/')
			if result[0:5] == 'C:/c/': return result.replace('C:/c/','C:/')
			return result
		else:
			return abspath(join(path, filename))

	@classmethod
	def get_filenm(self, path):
		result = self.stdpath(path)
		return result.split('/')[-1]

	@classmethod
	def get_prefix(self, filepath):
		return os.path.splitext(filepath)[0]

	@classmethod
	def get_extension(self, filepath):
		return os.path.splitext(filepath)[-1]

	@classmethod
	def get_folders(self, wd, fix=True):

		dpaths = []

		for root, dirname, _ in os.walk(wd):
			for d in dirname:
				dpath = self.joinpath(root,d)
				if isdir(dpath):
					if ' ' in d and fix == True:
						npath = os.rename(dpath,self.joinpath(root,d.replace(' ','_')))
						dpaths.append(npath)
					else:
						dpaths.append(dpath)

		return dpaths

	@classmethod
	def get_files(self, wd, extension=None, fix=True):

		filepaths = []

		for root, _ , files in os.walk(wd):
			for f in files:
				filepath = self.joinpath(root,f)
				if isfile(filepath):
					if ' ' in f and fix == True:
						npath = os.rename(filepath,self.joinpath(root,f.replace(' ','_')))
						filepaths.append(npath)
					else:
						filepaths.append(filepath)

		if extension == None or len(filepaths) == 0:
			return filepaths

		result = []

		for filepath in filepaths:
			if self.get_extension(filepath) == extension:
				result.append(filepath)

		return result

	@classmethod
	def _get_last_line(self, openedFile):
		try:
			openedFile.seek(-2, os.SEEK_END)
			while openedFile.read(1) != b'\n':
				openedFile.seek(-2, os.SEEK_CUR)
		except:
			pass

	@classmethod
	def read_last_line(self, filepath):
		with open(filepath,'rb') as f:
			self._get_last_line(f)
			last_line = '\n' + f.readline().decode(errors='replace')
			return last_line

	@classmethod
	def read_line(self, filepath, linenum):
		with open(filepath,'r') as f:
			for n, line in enumerate(f):
				if n == linenum:
					return line.rstrip('\n')
		return None

	@classmethod
	def read_file(self, filepath):
		lines = []
		with open(filepath,'r') as f:
			for line in f:
				if line[0] != '#':
					lines.append(line.rstrip('\n'))
		return lines



from os.path import isfile
from getpass import getpass
from ..utils.commoncmd import CommonCmd as cmd

class StashGUI:

	def __init__(self, scrambler, instance, configparser):
		self.scrambler = scrambler
		self.instance = instance
		self.configparser = configparser

	def noconfig_warning(self):
		cmd.clear()
		print('Warning: Config file not found or empty. This is required to use Stash.')
		print(' ')
		print('Instructions on creating a config file:')
		print('1) Create your .config based off of .config-template')
		print('2) Place your .config file into {}'.format(self.configparser.rootpath))
		print('3) [Optional] Encrypt your .config file with this app (this will create .config-c)')
		print('4) [Optional] Delete your original .config file once .config-c is created')
		print(' ')
		print('See .config-template file as an example, located in:')
		print(self.configparser.rootpath)
		print(' ')
		print('Press any key to exit.')
		input(); cmd.clear(); return

	def optionscreen(self):
		if self.configparser.hasEncyptedTag() == False:
			print('Warning: Your .config does not appear to be encrypted! Be sure to encrypt it.')
		print(' ')
		print('What would you like to do?')
		print('[1] Stash files, [2] Retrieve stashed files, [3] Encrypt config file.')

	def option_enc_config(self):
		filepath = self.configparser.filepath
		if self.configparser.hasEncyptedTag() == True:
			cmd.clear()
			print('No action taken, .config-c appears to be encrypted already:')
			print(filepath)
			return
		print(' ')
		print('Are you sure you want to encrypt the following [y/n]:')
		print(filepath)
		confirm = input(); cmd.clear()
		if confirm != 'y': print('Exited, no action taken.'); return
		if isfile(filepath) != True: print('Invalid file, no action taken.'); return

		print('Encrypting: ' + filepath)
		print(' ')
		password = getpass('Password (>=16 chars required): ')
		if password == '': cmd.clear(); print('Password cannot be blank, no action taken.'); return
		if len(password) <= 15: 
			cmd.clear(); print('Password must be 16 characters or more, no action taken.')
			del password; return
		print(' ')
		confirm = getpass('Please confirm password: ')
		cmd.clear()
		if password != confirm: del password; del confirm; print('Password mismatch, no action taken.'); return

		print('Encryption started...')
		result = self.scrambler.encrypt_file(password,filepath,
							decrypt=False,keep_org=False,naked=False)
		print(result['message'])
		print(' ')
		print('Encryption complete.')
		print(' ')
		del password; del confirm
		input(); cmd.clear(); return

	def option_stash(self, inverse=False):
		"""
		Data format:
		{"origin_dir": "/home/origin_directory/",
		"stash_dir": "/home/stash_directory/",
		"stash_key": {
		"filename1": "stashed_filename1",
		"filename2": "stashed_filename2",
		"filename3": "stashed_filename3"}}
		"""
		print(' ')
		if self.configparser.hasEncyptedTag():
			print('Please enter the password to the .config-c file.')
			print(' ')
			password = getpass('Password: ')
			print(' ')
			if password == '': cmd.clear(); print('Password cannot be blank, no action taken.'); return
		else:
			password = None

		parser = self.configparser.parse(password)
		if parser['status'] != 200:
			cmd.clear(); print('Attempting to parse config file...')
			print(' ')
			if parser['status'] == 405:
				print('Error: config file formatted incorrectly.')
				print('Your config file should be formatted like .config-template before encrypting it.')
			else:
				print(parser['message'])
			print(' '); print('Press any key to exit.')
			del password; input(); cmd.clear()
			print('Exited. No action taken.')
			return

		data = parser['output']
		if self.scrambler.invalid_stash_data(data):
			cmd.clear(); print('Attempting to parse config file...')
			print(' '); print('Error: config file formatted incorrectly.')
			print('Your config file should be formatted like .config-template before encrypting it.')
			print(' '); print('Press any key to exit.')
			del password; input(); cmd.clear()
			print('Exited. No action taken.')
			return

		if password != None: cmd.clear(); print('Password to .config-c accepted!'); print(' ')
		if inverse == True:
			keyword = 'Retrieve'
			print('Are you sure you want to retrieve your stashed files to the following directory? [y/n]')
		else:
			keyword = 'Stash'
			print('Are you sure you want to stash files located in the following directory? [y/n]')
		print(data['origin_dir'])
		confirm = input()
		if confirm != 'y': cmd.clear(); print('Exited, no action taken.'); return

		cmd.clear()
		print('{} started...'.format(keyword))

		response = self.scrambler.stash_all(data, inverse=inverse)
		if response['status'] != 200:
			print(' '); print(response['message'])
			print(' '); print('Press any key to exit.')
			del password; input(); cmd.clear()
			print('Exited. No action taken.')
			return

		for output in response['output']:
			print(output)

		print(' ')
		print(response['message'])
		input(); cmd.clear(); return

	def run(self):
		cmd.clear()
		load = self.configparser.load_config()
		if load['status'] != 200:
			self.noconfig_warning()
			print('Exited. No action taken.')
			return

		self.optionscreen()
		select = input()

		if select not in ('1','2','3'):
			#'[1] Stash files, [2] Retrieve stashed files, [3] Encrypt config file.'
			cmd.clear()
			print('Invalid selection. Try again.')
			return

		if select == '1':
			self.option_stash()
			return

		if select == '2':
			self.option_stash(inverse=True)
			return

		if select == '3':
			self.option_enc_config()
			return


import os
from .dircrawler import DirCrawler as dc

class FileModder:

	@classmethod
	def add_msg(self, filepath, msg):
		indent_msg = '\n' + msg
		if dc.read_last_line(filepath) != indent_msg:
			with open(filepath,'a') as f:
				f.write(indent_msg)
			return 'Added message: ' + filepath
		else:
			return 'No message to add: ' + filepath

	@classmethod
	def rm_msg(self, filepath, msg):
		indent_msg = '\n' + msg
		if dc.read_last_line(filepath) == indent_msg:
			with open(filepath,'rb+') as f:
				dc.get_last_line(f)
				f.truncate()
			with open(filepath,'rb+') as f:
				dc.get_last_line(f)
				f.truncate()
			return 'Removed message: ' + filepath
		else:
			return 'No message to remove: ' + filepath

	@classmethod
	def write_msg_all(self, wd, msg='secret-msg', extension=None, remove=True):
		filepaths = dc.get_files(wd, extension=extension)

		if len(filepaths) <= 0:
			return 'No files found.'

		for filepath in filepaths:

			print(self.add_msg(filepath, msg=msg))

			if remove == True:
				print(self.rm_msg(filepath, msg=msg))

		return 'Write message complete.'

	@classmethod
	def add_extension(self, wd, extension):
		if extension == None or '.' not in extension:
			return 'No extension found.'

		filepaths = dc.get_files(wd, extension=None)

		if len(filepaths) <= 0:
			return 'No files found.'

		for filepath in filepaths:
			if '.' not in filepath:
				os.rename(filepath, filepath+extension)
				print('Renamed: ' + str(filepath))

		return dc.get_files(wd, extension)

	@classmethod
	def add_tag(self, filepath, newtag, curr_tag_options=[], new_tag_options=[]):

		extension = dc.get_extension(filepath)
		prefix = dc.get_prefix(filepath)

		current_tag = '-' + prefix.split('-')[-1]

		if current_tag in new_tag_options:
			return filepath

		if current_tag in curr_tag_options:
			return prefix.split(current_tag)[0] + newtag + extension
		else:
			return prefix + newtag + extension



import os
from .dircrawler import DirCrawler as dc

class Instance:

	def __init__(self):
		self.wd = None

	def clear_wd(self):
		self.wd = None
		return {'status': 200, 'message': 'Working directory cleared.'}

	def set_wd(self,raw_wd):
		if raw_wd == '': return {'status': 400,
					'message': 'Invalid input, no action taken.'}
		wd = dc.stdpath(raw_wd)
		if os.path.isdir(wd) == False:
			return {'status': 400, 'message': 'Invalid input, no action taken.'}
		self.wd = wd
		os.chdir(wd)
		return {'status': 200, 'message': 'Working directory set: {}'.format(wd)}

	def set_cwd_as_wd(self):
		curr_dir = os.getcwd()
		wd = dc.stdpath(curr_dir)
		if os.path.isdir(wd) == False:
			return {'status': 400, 'message': 'Invalid input, no action taken.'}
		self.wd = wd
		return {'status': 200, 'message': 'Working directory set: {}'.format(wd)}



import os; import sys; import json
from .dircrawler import DirCrawler as dc
from .encryption import OpenSSLEncyptor as ossl

class ConfigParser:

	def __init__(self):
		self.build_loc = 'ScramblerApp/config'
		self.dev_loc = 'config'
		self.path_type = None
		self.rootpath = None
		self.filepath = None
		self.filename = None
		self.load_config()

	def _find(self, filename):
		build_rootpath = dc.joinpath(sys.prefix, self.build_loc)
		build_filepath = dc.joinpath(build_rootpath, filename)
		build_exists = os.path.exists(build_filepath)

		dev_rootpath = dc.joinpath(os.path.dirname(
			os.path.dirname(os.path.dirname(os.path.dirname(__file__)))),
			self.dev_loc)
		dev_filepath = dc.joinpath(dev_rootpath, filename)
		dev_exists = os.path.exists(dev_filepath)

		if build_exists == True:
			self.path_type = 'build'
			self.rootpath = build_rootpath
			self.filepath = build_filepath
			self.filename = filename
		elif dev_exists == True:
			self.path_type = 'dev'
			self.rootpath = dev_rootpath
			self.filepath = dev_filepath
			self.filename = filename
		else:
			self.path_type = 'build'
			self.rootpath = build_rootpath
			self.filepath = None
			self.filename = None

	def _find_config_file(self):
		possible_names = ['.config-c', '.config', '.config-d', '.config-NAKED']
		for name in possible_names:
			self._find(name)
			if self.filepath != None:
				return {'status': 200, 'message': 'Find config complete.'}
		return {'status': 400, 'message': 'Error: no config found.'}

	def hasContent(self):
		if self.filepath == None: return False
		if os.stat(self.filepath).st_size > 0:
			return True
		else:
			return False

	def hasEncyptedTag(self):
		if self.filename == '.config-c':
			return True
		else:
			return False

	def load_config(self):
		self._find_config_file()
		has_content = self.hasContent()
		if has_content == True: 
			return {'status': 200,
					'message': 'Load config file complete.'}
		else:
			return {'status': 400,
					'message': 'Error: config file not found or empty.'}

	def parse(self, password=None):
		has_enctag = self.hasEncyptedTag()

		try:
			file_content = ''.join(dc.read_file(self.filepath))
		except:
			return {'status': 400,
						'message': 'Error: config file could not be read.', 'output': None}

		if has_enctag == True and password == None:
			return {'status': 400,
				'message': 'Error: config file requires password to read.', 'output': None}

		if has_enctag == True and password != None:
			data = {'medium': 'text', 'input': file_content, 'outpath': None}
			response = ossl.encrypt(password, data, decrypt=True)

			if response['status'] != 200: return response

			try:
				result = json.loads(response['output'])
				return {'status': 200, 'message': 'Read config file complete.',
						'output': result}
			except:
				return {'status': 400,
					'message': response['message'],
					'output': None}

		if has_enctag == False:
			try:
				result = json.loads(file_content)
				return {'status': 200, 'message': 'Read config file complete.',
						'output': result}
			except:
				return {'status': 400,
					'message': 'Error: config file not structured correctly.',
					'output': None}


import os; import random;
import shlex; import subprocess
from os.path import isfile, isdir, exists
from .utils.dircrawler import DirCrawler as dc
from .utils.encryption import OpenSSLEncyptor as ossl
from .utils.filemodder import FileModder as fm
from .utils.commoncmd import CommonCmd as cmd

class Scrambler:

	def formatted_ext(self, raw_extension):
		if raw_extension == '*':
			return None
		if raw_extension == '':
			return '.txt'
		if raw_extension[0] != '.':
			return '.' + raw_extension
		else:
			return raw_extension

	def timetravel(self,path,hours_ago=None):
		if hours_ago == None: hours_ago = random.randrange(1,121000)

		pathlex = shlex.quote(path)
		hourslex = shlex.quote(str(hours_ago))
		command = shlex.split('touch -d "{h} hours ago" {p}'.format(h=hourslex,p=pathlex))
		process = subprocess.run(command,
					stdout=subprocess.PIPE,stderr=subprocess.PIPE)

		returncode = process.returncode

		if returncode == 0:
			return {'status': 200, 'message': 'Timetravelled: ' + path}
		else:
			return {'status': 400, 'message': 'Error timetravelling: ' + path}

	def timetravel_files(self, dir, extension=None):
		files = dc.get_files(dir, extension=extension)

		if len(files) <= 0: return {'status': 400, 'message': 'Error timetravelling: no files found.', 'output': []}

		output = [self.timetravel(f)['message'] for f in files]
		return {'status': 200, 'message': 'Timetravel files complete.', 'output': output}

	def timetravel_folders(self, dir):
		folders = dc.get_folders(dir)

		if exists(dir) != True: return {'status': 400, 'message': 'Error timetravelling: no directory found.', 'output': []}

		if len(folders) <= 0:
			output = ['No folders found to timetravel, no action taken.']
		else:
			output = [self.timetravel(folder)['message'] for folder in folders]

		output.append(self.timetravel(dir)['message'])

		return {'status': 200, 'message': 'Timetravel folders complete.', 'output': output}

	def invalid_stash_data(self, data):
		no_origin_dir = 'origin_dir' not in data
		no_stash_dir = 'stash_dir' not in data
		no_stash_key = 'stash_key' not in data
		if no_origin_dir or no_stash_dir or no_stash_key:
			return True
		if type(data['origin_dir']) != str or type(data['stash_dir']) != str:
			return True
		if type(data['stash_key']) != dict:
			return True
		stash_key = data['stash_key']
		keys = list(stash_key.keys())
		values = list(stash_key.values())
		if len(keys) == 0 or len(values) == 0: return True
		if len(keys) != len(values): return True
		return False

	def _censor_stash_errors(self, errormsg, sensitiveinfo, censored=True):
		if censored == True:
			return errormsg + '. Please check your config file.'
		else:
			return errormsg + ' ' + sensitiveinfo

	def stash(self, curr_filename, curr_dir, new_filename, new_dir, overwrite=False, remove=False, censored=True):
		curr_filepath = dc.joinpath(curr_dir, curr_filename)
		if exists(curr_filepath) != True: return {'status': 400,
					'message': self._censor_stash_errors(
						'Error: stash file was not found', curr_filepath, censored)}

		if isfile(curr_filepath) != True: return {'status': 400,
					'message': self._censor_stash_errors(
						'Error: invalid stash file', curr_filepath, censored)}

		new_filepath = dc.joinpath(new_dir, new_filename)
		if exists(new_filepath) == True and overwrite == False: return {'status': 400,
					'message': self._censor_stash_errors(
						'Error: stash file already exists', new_filepath, censored)}

		response = cmd.copyfile(curr_filepath,new_filepath)
		if response['status'] != 200: return response
		self.timetravel(new_filepath)
		if remove == True: 
			try:
				os.remove(curr_filepath)
			except:
				return {'status': 400, 'message': self._censor_stash_errors(
						'Error: failed to remove stash file', curr_filepath, censored)}
		return response

	def stash_all(self, data, inverse=False):
		"""
		Data format:
		{"origin_dir": "/home/origin_directory/",
		"stash_dir": "/home/stash_directory/",
		"stash_key": {
		"filename1": "stashed_filename1",
		"filename2": "stashed_filename2",
		"filename3": "stashed_filename3"}}
		"""
		result = {'status': None, 'message': None, 'output': []}
		origin_dir = data['origin_dir']
		stash_dir = data['stash_dir']

		if isdir(origin_dir) == False: return {'status': 400,
				'message': 'Error: could not find origin directory {}'.format(origin_dir), 'output': []}

		if isdir(stash_dir) == False: return {'status': 400,
				'message': 'Error: could not find stash directory. Please check your config file.', 'output': []}

		stash_key = data['stash_key']
		keys = list(stash_key.keys())
		values = list(stash_key.values())
		statuscodes = []

		for i in range(len(keys)):
			if keys[i] == '' or values[i] == '': return {'status': 400,
				'message': 'Error: Invalid config file, keys and values cannot be blank. Please check your config file.',
				'output': []}

		for i in range(len(keys)):
			if inverse == True:
				response = self.stash(values[i],stash_dir,keys[i],origin_dir,overwrite=False,remove=False)
			else:
				response = self.stash(keys[i],origin_dir,values[i],stash_dir,overwrite=True,remove=True)
			statuscodes.append(response['status'])
			result['output'].append(response['message'])

		if inverse == True:
			keyword = 'Retrieve'
			if len(statuscodes) > 1 and 200 not in statuscodes:
				result['status'] = 400
				result['message'] = 'Error: no stash files retrieved. Please check your config file.'
				return result
		else:
			keyword = 'Stash'
			if len(statuscodes) > 1 and 200 not in statuscodes:
				result['status'] = 400
				result['message'] = 'Error: no stash files found in {}'.format(origin_dir)
				return result
			timetravel = self.timetravel(stash_dir)
			if timetravel['status'] == 200:
				result['output'].append('Timetravel completed for stash directory.')
			else:
				result['output'].append('Error timetravelling stash directory.')

		result['status'] = 200
		result['message'] = '{} completed.'.format(keyword)
		return result

	def encrypt_msg(self, password, message, decrypt=False):
		data = {'medium': 'text', 'input': message, 'outpath': None}
		result = ossl.encrypt(password, data, decrypt)
		del password; return result

	def encrypt_file(self, password, filepath,
					decrypt=False, keep_org=False, naked=False):
		result = {'status': None, 'message': None}
		tag_options = {'encrypt' : ['-c'],
			'decrypt' : ['-d', '-NAKED']}

		if decrypt == True:
			index = 1 if naked == True else 0
			outpath = fm.add_tag(filepath,tag_options['decrypt'][index],
								tag_options['encrypt'],tag_options['decrypt'])
		else:
			outpath = fm.add_tag(filepath,tag_options['encrypt'][0],
								tag_options['decrypt'],tag_options['encrypt'])

		if exists(filepath) == False:
			result['status'] = 400
			result['message'] = 'Error failed to find file: ' + filepath
			del password; return result

		if exists(outpath) == True:
			result['status'] = 400
			result['message'] = 'Error output path already exists: ' + outpath
			del password; return result

		if outpath == filepath:
			result['status'] = 400
			result['message'] = 'Action already performed on: ' + filepath
			del password; return result

		data = {'medium': 'file', 'input': filepath, 'outpath': outpath}
		response = ossl.encrypt(password, data, decrypt)
		if response['status'] == 400:
			try:
				os.remove(outpath)
			except:
				pass
			del password; return response

		self.timetravel(outpath)

		if keep_org == True:
			result['status'] = response['status']
			result['message'] = response['message'] + ' (original retained).'
			del password; return result

		try:
			os.remove(filepath)
			result['status'] = 200
			result['message'] = response['message'] + ' (original deleted).'
		except:
			result['status'] = 400
			result['message'] = response['message'] + ' (Error deleting original).'

		del password; return result

	def encrypt_all_files(self, password, wd, extension=None,
					decrypt=False,keep_org=False,naked=False):
		filepaths = dc.get_files(wd, extension=extension)
		if len(filepaths) <= 0: del password; return {'status': 400, 'message': 'Error: No files found.', 'output': []}

		output = [self.encrypt_file(password,filepath,
					decrypt=decrypt,keep_org=keep_org,naked=naked)['message'] for filepath in filepaths]

		timetravel = self.timetravel_folders(wd)
		if timetravel['status'] != 200 or len(timetravel['output']) == 0:
			output.append('Error timetravelling folders, timetravelling skipped.')
		else:
			for o in timetravel['output']: output.append(o)

		del password; return {'status': 200, 'message': 'Encrypt all files complete.', 'output': output}

class ScramblerGUI:

	def __init__(self, scrambler, instance, encryptiongui, stashgui):
		self.scrambler = scrambler
		self.instance = instance
		self.encryptiongui = encryptiongui
		self.stashgui = stashgui

	def splashscreen(self):
		cmd.clear()
		print('Welcome to the Scrambler!')

	def optionscreen(self):
		print(' ')
		print('What would you like to do?')
		print('(s) Set Dir, (e) Encrypt, (d) Decrypt, (st) Stash, (t) Timetravel, (q) Quit')

	def comingsoon(self):
		cmd.clear()
		print('Feature not yet available, no action taken.')

	def option_s(self):
		print(' ')
		print('What directory do you want to set as your working directory?')
		raw_wd = input()
		cmd.clear()
		setwd = self.instance.set_wd(raw_wd)
		print(setwd['message'])
		return

	def option_t(self):
		cmd.clear()
		if self.instance.wd == None:
			print('Error: No working directory set. Please set working directory first.'); return

		print('Warning: You are about to timetravel all files and folders in the following directory and its subdirectories.')
		print('Timetravel will alter the metadata to a scrambled date/time in the past of everything in:')
		print(self.instance.wd)
		print(' ')
		print('You may specify a file type to timetravel. Leave blank for default .txt files.')
		print('Use * to timetravel all files regardless of type (this can be dangerous).')
		print(' ')
		raw_extension = input('Specify a file type [Optional]: ')
		extension = self.scrambler.formatted_ext(raw_extension)
		print(' ')
		if extension == None:
			confirm = input('Are you sure you want to timetravel all folders and files of all types [y/n]: ')
		else:
			confirm = input('Are you sure you want to timetravel all folders and files with extension {} [y/n]: '.format(extension))
		cmd.clear()
		if confirm != 'y': print('Exited, no action taken.'); return
		if isdir(self.instance.wd) != True: print('Invalid path, no action taken.'); return

		print('Timetravel started...')
		tt_files = self.scrambler.timetravel_files(self.instance.wd, extension=extension)
		if tt_files['status'] != 200 or len(tt_files['output']) == 0:
			print('No files found to timetravel, no action taken.')
		else:
			for f in tt_files['output']: print(f)

		tt_folders = self.scrambler.timetravel_folders(self.instance.wd)
		if tt_folders['status'] != 200 or len(tt_folders['output']) == 0:
			print('No folders found to timetravel, no action taken.')
		else:
			for folder in tt_folders['output']: print(folder)

		print(' ')
		print('Timetravel complete.')
		input(); cmd.clear(); return

	def option_pwd(self):
		cmd.clear()
		if self.instance.wd == None:
			print('Error: No working directory set. Please set working directory first.'); return
		else:
			print('Working directory: {}'.format(cmd.pwd())); return

	def option_ls(self):
		cmd.clear()
		if self.instance.wd == None:
			print('Error: No working directory set. Please set working directory first.'); return

		ls = cmd.ls()

		if len(ls) == 0:
			print('Working directory is empty.'); return
		else:
			print(' '.join(ls)); return

	def run(self):
		cmd.clear()
		self.splashscreen()

		while True:
			self.optionscreen()
			select = input()

			if select not in ('q','pwd','ls','s','e','d','st','t','q'):
				#'(s) Set Dir, (e) Encrypt, (d) Decrypt, (st) Stash, (t) Timetravel, (q) Quit'
				cmd.clear(); print('Invalid selection. Try again.')

			if select == 'q':
				cmd.clear()
				break

			if select == 'pwd':
				self.option_pwd()

			if select == 'ls':
				self.option_ls()

			if select == 's':
				self.option_s()

			if select == 'e':
				self.encryptiongui.run(decrypt=False)

			if select == 'd':
				self.encryptiongui.run(decrypt=True)

			if select == 'st':
				self.stashgui.run()

			if select == 't':
				self.option_t()



from os.path import isfile, isdir
from getpass import getpass
from ..utils.commoncmd import CommonCmd as cmd

class EncryptionGUI:

	def __init__(self, scrambler, instance):
		self.scrambler = scrambler
		self.instance = instance

	def optionscreen(self, decrypt=False):
		if decrypt == True:
			keyword = 'DECRYPT'
		else:
			keyword = 'ENCRYPT'

		print('What would you like to {}?'.format(keyword))
		print('[1] A Message, [2] A File, [3] All Files, [4] Columns in a Dataframe')

	def option_enc_msg(self, decrypt=False):
		if decrypt == True:
			keyword = 'Decrypt'
			pass_msg = 'Password: '
		else:
			keyword = 'Encrypt'
			pass_msg = 'Password (>=16 chars required): '
		cmd.clear()
		print('What is the message you want to {}?'.format(keyword.upper()))
		print(' ')
		message = input('Input your message: ')
		if message == '': cmd.clear(); print('Exited, no action taken.'); return
		print(' ')

		password = getpass(pass_msg)
		if password == '':
			cmd.clear(); print('Password cannot be blank, no action taken.'); return

		if decrypt == True:
			confirm = password
		else:
			if len(password) <= 15:
				cmd.clear(); print('Password must be 16 characters or more, no action taken.')
				del password; return
			print(' ')
			confirm = getpass('Please confirm password: ')

		cmd.clear()
		if password != confirm: del password; del confirm; print('Password mismatch, no action taken.'); return

		result = self.scrambler.encrypt_msg(password,message,decrypt)
		if result['status'] != 200: print(result['message']); return

		print('{}ion complete.'.format(keyword))
		print(' ')

		if decrypt == True:
			print('secret: {}'.format(result['output']))
		else:
			print('cipher: {}'.format(result['output']))
		print(' ')
		print('Press any key to exit.')
		del password; del confirm
		input(); cmd.clear(); return

	def option_enc_file(self, decrypt, keep_org, naked=False, all=False):
		wd = self.instance.wd

		if decrypt == True:
			keyword = 'decrypt'
			keywording = 'Decrypting'
			keywordion = 'Decryption'
			pass_msg = 'Password: '
		else:
			keyword = 'encrypt'
			keywording = 'Encrypting'
			keywordion = 'Encryption'
			pass_msg = 'Password (>=16 chars required): '

		if wd == None: cmd.clear(); print('Error: No working directory set. Please set working directory first.'); return

		if all == True:
			cmd.clear()
			print('Warning: You are about to {} all files in the following directory and its subdirectories.'.format(keyword.upper()))
			print('This will OVERWRITE the original files. Make sure to backup the contents of this directory:')
			print(wd)
			print(' ')
			print('You may specify a file type to {}. Leave blank for default .txt files.'.format(keyword))
			print('Use * to {} all files regardless of type (this can be dangerous).'.format(keyword))
			print(' ')
			raw_extension = input('Specify a file type [Optional]: ')
			extension = self.scrambler.formatted_ext(raw_extension)
			print(' ')
			if extension == None:
				confirm = input('Are you sure you want to {} all files of all types [y/n]: '.format(
					keyword))
			else:
				confirm = input('Are you sure you want to {} all files with extension {} [y/n]: '.format(
					keyword,extension))
			cmd.clear()
			if confirm != 'y': print('Exited, no action taken.'); return
			if isdir(wd) != True: print('Invalid path, no action taken.'); return
			if extension == None:
				print(keywording + ' all files in: ' + wd)
			else:
				print(keywording + ' all ' + extension + ' in: ' + wd)
		else:
			print(' ')
			print('Which file do you want to {}?'.format(keyword.upper()))
			filename = input()
			if filename == '': cmd.clear(); print('Exited, no action taken.'); return
			print(' ')
			print('Are you sure you want to {} the following [y/n]:'.format(keyword))
			print(filename)
			confirm = input(); cmd.clear()
			if confirm != 'y': print('Exited, no action taken.'); return
			if isfile(filename) != True: print('Invalid file, no action taken.'); return
			print(keywording + ': ' + filename)

		print(' ')
		password = getpass(pass_msg)
		if password == '': cmd.clear(); print('Password cannot be blank, no action taken.'); return

		if decrypt == True:
			confirm = password
		else:
			if len(password) <= 15: cmd.clear(); print('Password must be 16 characters or more, no action taken.'); return
			print(' ')
			confirm = getpass('Please confirm password: ')

		cmd.clear()
		if password != confirm: del password; del confirm; print('Password mismatch, no action taken.'); return

		print('{} started...'.format(keywordion))

		if all == True:
			result = self.scrambler.encrypt_all_files(password,wd,extension=extension,
							decrypt=decrypt,keep_org=keep_org,naked=naked)
			if result['status'] != 200:
				print(result['message'])
			else:
				for output in result['output']: print(output)
		else:
			result = self.scrambler.encrypt_file(password,filename,
							decrypt=decrypt,keep_org=keep_org,naked=naked)
			print(result['message'])

		print(' ')
		print('{} complete.'.format(keywordion))
		del password; del confirm
		input(); cmd.clear(); return

	def run(self, decrypt):
		cmd.clear()
		print(' ')
		self.optionscreen(decrypt)
		select = input()

		if select not in ('1','2','3','4'):
			#'[1] A Message, [2] A File, [3] All Files, [4] Columns in a Dataframe'
			cmd.clear()
			print('Invalid selection. Try again.')
			return

		if select == '1':
			self.option_enc_msg(decrypt=decrypt)
			return

		if select == '2':
			if decrypt == True:
				self.option_enc_file(decrypt=decrypt,keep_org=True,naked=True)
			else:
				self.option_enc_file(decrypt=decrypt,keep_org=True)
			return

		if select == '3':
			if decrypt == True:
				self.option_enc_file(decrypt=decrypt,keep_org=False,all=True)
			else:
				self.option_enc_file(decrypt=decrypt,keep_org=False,all=True)
			return

		if select == '4':
			cmd.clear()
			print('Feature not yet available, no action taken.')
			return


#!/usr/bin/python3 -B
from scramblerapp import *

def main():
	scrambler = Scrambler()
	instance = Instance()
	configparser = ConfigParser()
	encryptiongui = EncryptionGUI(scrambler, instance)
	stashgui = StashGUI(scrambler, instance, configparser)
	scramblergui = ScramblerGUI(scrambler, instance, encryptiongui, stashgui)
	scramblergui.run()

if __name__ == '__main__':
	raise SystemExit(main())


# Exports package resources so users can use in their own project.
__all__ = [
	'Scrambler', 'ScramblerGUI',
	'EncryptionGUI', 'StashGUI',
	'Instance', 'ConfigParser'
]

from .scrambler import *
from .gui.encryptiongui import *
from .gui.stashgui import *
from .utils.instance import *
from .utils.configparser import *


import os;
import shlex; import subprocess
from os.path import isfile, isdir
from .dircrawler import DirCrawler as dc

class CommonCmd:

	@classmethod
	def pwd(self):
		command = 'pwd'
		process = subprocess.run(command,
			stdout=subprocess.PIPE,stderr=subprocess.PIPE)
		returncode = process.returncode
		result = dc.stdpath(process.stdout.decode().rstrip('\n'))
		if returncode == 0:
			return result
		else:
			return 'Error unable to print working directory.'

	@classmethod
	def ls(self):
		files = [f for f in os.listdir('.') if isfile(f)]
		folders = [d for d in os.listdir('.') if isdir(d)]
		files.sort()
		folders.sort()
		return folders + files

	@classmethod
	def clear(self):
		os.system('cls' if os.name=='nt' else 'clear')

	@classmethod
	def copyfile(self,curr_filepath,new_filepath):

		cfilelex = shlex.quote(curr_filepath)
		nfilelex = shlex.quote(new_filepath)

		command = shlex.split('cp {c} {n}'.format(c=cfilelex,n=nfilelex))
		process = subprocess.run(command,
			stdout=subprocess.PIPE,stderr=subprocess.PIPE)
		returncode = process.returncode
		if returncode == 0:
			return {'status': 200, 'message': 'File created: ' + new_filepath}
		else:
			return {'status': 400, 'message': 'Error creating file for: ' + curr_filepath}


# Allows execution on calling the name the directory: python3 -m <packagename>
# See: https://stackoverflow.com/questions/4042905/what-is-main-py
from scramblerapp.main import main

if __name__ == '__main__':
	raise SystemExit(main())


import shlex; import subprocess

class OpenSSLEncyptor:

	@classmethod
	def encrypt(self, password, data, decrypt=False):
		"""
		Example data inputs:
		data = {'medium': 'file', 'input': '/filepath/example', 'outpath': '/example-c'}
		data = {'medium': 'text', 'input': 'hello world', 'outpath': None}
		"""
		medium_options = ('file', 'text')
		result = {'status': None, 'message': None, 'output': None}
		if type(data) != dict: del password; return {'status': 400,
								'message': 'Error: Input data incorrect format.'}
		if data['medium'] not in medium_options: del password; return {'status': 400,
								'message': 'Error: Invalid medium.'}

		passwd = shlex.quote(password)
		dashd = ' -d' if decrypt == True else ''
		keyword = 'Decrypt' if decrypt == True else 'Encrypt'
		keywording = 'decrypting' if decrypt == True else 'encrypting'

		if data['medium'] == 'text':
			inputtext = data['input']
			if "'" in inputtext or '"' in inputtext:
				del passwd; del password; return {'status': 405,
				'message': 'Error: Quote characters not allowed.'}
			text = shlex.quote(data['input'])
			pipetext = shlex.split('echo "{t}"'.format(t=text))
			command = shlex.split('openssl aes-256-cbc{d} -a -salt -pbkdf2 -k {p}'.format(
						d=dashd,p=passwd))

			try:
				pipe = subprocess.Popen(pipetext, stdout=subprocess.PIPE)
				process = subprocess.check_output(command, stdin=pipe.stdout,
							stderr=subprocess.DEVNULL)
				pipe.wait()
				raw_output = process.decode()
				clean1 = raw_output[:-1] if raw_output[-1] == "\n" else raw_output
				clean2 = clean1[:-1] if clean1[-1] == "\r" else clean1
				clean3 = clean2[:-1] if clean2[-1] == "'" else clean2
				final_clean = clean3[1:] if clean3[0] == "'" else clean3

				result['status'] = 200
				result['message'] = '{} completed.'.format(keyword)
				result['output'] = final_clean
				del passwd; del password; return result
			except:
				result['status'] = 400
				result['message'] = 'Error: {}ion failed. Make sure your password is correct.'.format(keyword)
				del passwd; del password; return result

		if data['medium'] == 'file':
			try:
				filepath = shlex.quote(data['input'])
				outpath = shlex.quote(data['outpath'])
			except:
				del passwd; del password; return {'status': 200, 'message': 'Error: Invalid medium.'}

			c = 'openssl aes-256-cbc{d} -a -salt -pbkdf2 -in {f} -out {o} -k {p}'.format(
							d=dashd,f=filepath,o=outpath,p=passwd)
			command = shlex.split(c)
			process = subprocess.run(command,
							stdout=subprocess.PIPE,stderr=subprocess.PIPE)
			#stdout = process.stdout.decode() #to see error code
			returncode = process.returncode

			if returncode == 0:
				result['status'] = 200
				result['message'] = '{}ed: '.format(keyword) + outpath
				del passwd; del password; return result
			else:
				result['status'] = 400
				result['message'] = 'Error {}: '.format(keywording) + filepath
				del passwd; del password; return result